/************************************************************************************
Blackship RT-Component
Copyright (c) 2010, Segway-Japan, Ltd.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/
#ifndef BLACKSHIP_H
#define BLACKSHIP_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "bs_if.h"
#include "intellirobotStub.h"

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;





/*!
  @brief Blackship I-O RTM-Module class\n
  inport TimedLong 'Speed*' [-1000 - 1000] is sended to Blackship\n
  inport TimedDouble 'Speed_mps' (m/s)\n
  inport TimedDouble 'Turn_rps'  (rad/s)\n
  inport TimedVelocity (m/s rad/s)
  outport TimedLong 'Encoder*' \n
  outport TimedOdometry 'Odometry' is internal odometry x(m) y(m) theta(rad) \n

\n
   configuration
\n
   ===use Blackship device port===
   str_port:std::string:/dev/ttyS0\n



*/
class Blackship
  : public RTC::DataFlowComponentBase
{
 public:
  Blackship(RTC::Manager* manager);
  ~Blackship();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry()
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  double m_Kspeed;
  double m_Kturn;
  double m_wheelwidth;
  double m_wheelradiusL;
  double m_wheelradiusR;
  double m_wheelcount1rot;
  std::string m_str_port;
  double m_Kodoxy;
  double m_Kodotheta;
  double m_Kslip;
  double m_mps2bs;
  int m_ODOoutput;
  int m_simulator;
  int m_idIIS;
  double m_startxIIS;
  double m_startyIIS;
  double m_startthetaIIS;
  double m_MaxV;
  double m_MaxW;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedULong m_Input;
  InPort<TimedULong> m_InputIn;

  TimedVelocity m_Velocity;
  InPort<TimedVelocity> m_VelocityIn;
  IIS::TimedVelocity m_VelocityIIS;
  InPort<IIS::TimedVelocity> m_VelocityIISIn;
  IIS::TimedVelocity2D m_Velocity2DIIS;
  InPort<IIS::TimedVelocity2D> m_Velocity2DIISIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedLong m_EncoderL;
  OutPort<TimedLong> m_EncoderLOut;
  TimedLong m_EncoderR;
  OutPort<TimedLong> m_EncoderROut;
  TimedOdometry m_Odometry;
  OutPort<TimedOdometry> m_OdometryOut;
  TimedOdometry m_OdometryB;
  OutPort<TimedOdometry> m_OdometryBOut;
  IIS::TimedVelocity m_RobotVelIIS;
  OutPort<IIS::TimedVelocity> m_RobotVelIISOut;
  IIS::TimedPosition m_RobotPosIIS;
  OutPort<IIS::TimedPosition> m_RobotPosIISOut;
  IIS::TimedVelocity2D m_RobotVel2DIIS;
  OutPort<IIS::TimedVelocity2D> m_RobotVel2DIISOut;
  IIS::TimedPose2D m_RobotPose2DIIS;
  OutPort<IIS::TimedPose2D> m_RobotPose2DIISOut;
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;

  int Speed2[2];
  int PortFd;

  unsigned long Stop;

  int Speed;
  int Turn;
  bs_status Status;

#if RTM_MAJOR_VERSION >= 1 
  coil::TimeValue curtime;
#else
  ACE_Time_Value curtime;
#endif

};


extern "C"
{
  void BlackshipInit(RTC::Manager* manager);
};

#endif // BLACKSHIP_H
