/************************************************************************************
Blackship RT-Component
Copyright (c) 2010, Segway-Japan, Ltd.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/
#include "winterm.h"

#ifdef WIN32

//========================================
//fcntl.h
//========================================

int open(const char *pathname, int flags, mode_t mode)
{
	HANDLE h;
	h = CreateFile(pathname, flags, 0, NULL, OPEN_EXISTING, 0, NULL);
	return (int)h;
}

int close(int fd)
{
	HANDLE h = (HANDLE)fd;
	CloseHandle(h);
	return 0;
}

ssize_t write(int fd, const void *buf, size_t count)
{
	HANDLE h = (HANDLE) fd;
	DWORD writtenbytes;
	if (WriteFile(h, buf, (DWORD)count, &writtenbytes, NULL)) {
		return (ssize_t)writtenbytes;
	}
	return -1;
}

ssize_t read(int fd, void *buf, size_t count)
{
	HANDLE h = (HANDLE) fd;
	DWORD readbytes;
	if (ReadFile(h, buf, (DWORD)count, &readbytes, NULL)) {
		return (ssize_t)readbytes;
	}
	return -1;
}

//========================================
//termios.h
//========================================

int tcflush(int fd, int queue_selector)
{
	return 1;
}

int tcgetattr(int fd, struct termios *termios_p)
{
	HANDLE h = (HANDLE) fd;
	GetCommState(h, &termios_p->dcb);
	return 0;
}

int tcsetattr(int fd, int optional_actions, struct termios *termios_p)
{
	HANDLE h = (HANDLE) fd;
	SetCommState(h, &termios_p->dcb);
	return 0;
}


void cfmakeraw(struct termios *termios_p)
{
	termios_p->dcb.DCBlength			= sizeof(DCB);

	//setting
	termios_p->dcb.ByteSize				= 8;
	termios_p->dcb.Parity				= NOPARITY;
	termios_p->dcb.StopBits				= ONESTOPBIT;

	//RTS/CTS flow control
	termios_p->dcb.fOutxCtsFlow			= TRUE;
	termios_p->dcb.fRtsControl			= RTS_CONTROL_ENABLE;

	termios_p->dcb.fOutxDsrFlow			= FALSE;
	termios_p->dcb.fDtrControl			= DTR_CONTROL_DISABLE;
	termios_p->dcb.fDsrSensitivity		= FALSE;
	termios_p->dcb.fTXContinueOnXoff	= FALSE;
	termios_p->dcb.fOutX				= FALSE;
	termios_p->dcb.fInX					= FALSE;
	termios_p->dcb.fErrorChar			= FALSE;
	termios_p->dcb.fNull				= FALSE;
	termios_p->dcb.fAbortOnError		= FALSE;
	termios_p->dcb.fDummy2				= 0;
	termios_p->dcb.wReserved			= 0;
	termios_p->dcb.XonLim				= 0;
	termios_p->dcb.XoffLim				= 0;
	termios_p->dcb.XonChar				= 0;
	termios_p->dcb.XoffChar				= 0;
	termios_p->dcb.ErrorChar			= 0;
	termios_p->dcb.EofChar				= 0;
	termios_p->dcb.EvtChar				= 0;
	termios_p->dcb.wReserved1			= 0;

}

int cfsetispeed(struct termios *termios_p, speed_t speed)
{
	termios_p->dcb.BaudRate = speed;
	return 0;
}

int cfsetospeed(struct termios *termios_p, speed_t speed)
{
	termios_p->dcb.BaudRate = speed;
	return 0;
}

speed_t cfgetispeed(const struct termios *termios_p)
{
	return (speed_t)termios_p->dcb.BaudRate;
}

speed_t cfgetospeed(const struct termios *termios_p)
{
	return (speed_t)termios_p->dcb.BaudRate;
}

int fcntl(int fd, int cmd)
{
	return 0;
}

int fcntl(int fd, int cmd, long arg)
{
	return 0;
}

int poll(struct pollfd *fds, unsigned int nfds, int timeout)
{
	return 1;
}


//=======================================
//math.h
//=======================================
double rint(double r)
{
	return (double) ((int) r);
}



#endif


