/************************************************************************************
Blackship RT-Component
Copyright (c) 2010, Segway-Japan, Ltd.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/
#ifndef __WINTERM_H__
#define __WINTERM_H__

#ifdef WIN32
//================================================================
//this file is wrapping linux termios interfaces with windows API
//================================================================
#include <windows.h>


//typedef int size_t;
typedef int ssize_t;
typedef int speed_t;
typedef int mode_t;

#define EAGAIN	(-1)
#define EINTR	(-1)




//========================================
//fcntl.h
//========================================
#define O_RDWR		(GENERIC_READ|GENERIC_WRITE)
#define O_SYNC		(0)
#define O_NONBLOCK	(0)
#define S_IRUSR		(0)
#define S_IWUSR		(0)


int open(const char *pathname, int flags, mode_t mode);
int close(int fd);
ssize_t write(int fd, const void *buf, size_t count);
ssize_t read(int fd, void *buf, size_t count);




//========================================
//termios.h
//========================================
#define B110		CBR_110 
#define B300		CBR_300 
#define B600		CBR_600 
#define B1200		CBR_1200
#define B2400		CBR_2400
#define B4800		CBR_4800
#define B9600		CBR_9600
#define B14400		CBR_14400
#define B19200		CBR_19200
#define B38400		CBR_38400
#define B56000		CBR_56000
#define B57600		CBR_57600
#define B115200		CBR_115200
#define B128000		CBR_128000
#define B256000		CBR_256000

struct termios
{
	HANDLE h;
	DCB dcb;
};

#define TCIFLUSH	(0)
#define TCOFLUSH	(0)
#define TCIOFLUSH	(0)
#define TCSAFLUSH	(0)

int tcflush(int fd, int queue_selector); 
int tcgetattr(int fd, struct termios *termios_p); 
int tcsetattr(int fd, int optional_actions, struct termios *termios_p); 
void cfmakeraw(struct termios *termios_p); 
int cfsetispeed(struct termios *termios_p, speed_t speed);
int cfsetospeed(struct termios *termios_p, speed_t speed);
speed_t cfgetispeed(const struct termios *termios_p); 
speed_t cfgetospeed(const struct termios *termios_p); 

#define F_SETFL	(0)
#define F_GETFL (0)

int fcntl(int fd, int cmd);
int fcntl(int fd, int cmd, long arg);
//int fcntl(int fd, int cmd, struct flock *lock);




//========================================
//poll.h
//========================================
/*
#define POLLIN      0x0001    // There is data to read 
#define POLLPRI     0x0002    // There is urgent data to read 
#define POLLOUT     0x0004    // Writing now will not block 
#define POLLERR     0x0008    // Error condition (output only) 
#define POLLHUP     0x0010    // Hung up (output only) 
#define POLLNVAL    0x0020    // Invalid request: fd not open (output only) 
*/

#define POLLIN	(1)

struct pollfd 
{
	int   fd;         /* file descriptor */
	short events;     /* requested events */
	short revents;    /* returned events */
};

struct nfds_t
{
	int dummy;
};

int poll(struct pollfd *fds, unsigned int nfds, int timeout);




//=======================================
//math.h
//=======================================
double rint(double r);




#endif//WIN32
#endif
